from itertools import product
from collections import namedtuple

import xarray as xr

from toargridding.toar_rest_client import AnalysisService
from toargridding.grids import GridDefinition
from toargridding.metadata import Metadata, TimeSample

GriddedResult = namedtuple("GriddedResult", ["dataset", "metadata"])


def get_gridded_toar_data(
    analysis_service: AnalysisService,
    grid: GridDefinition,
    time: TimeSample,
    variables: list[str],
    stats: list[str],
    **kwargs
) -> tuple[list[xr.Dataset], list[Metadata]]:
    """ API to download data as xarrays
    
    The function creates all combinations of the variable and stats list

    Parameters:
    ----------
    analysis_service:
        access to the REST API and manages download of data
    grid:
        grid for the output data.
    time:
        sampled values in time
    variables:
        list of variables to be extracted from the TOARdatabase
    stats:
        list of statistical properties to be extracted from the TOAR database

    return:
    -------
        Gridded datasets for each combination of variables and stats and appropriate metadata for each dataset.
    """

    metadatas = [
        Metadata.construct(standard_name=var, time=time, stat=stat, moreOptions=kwargs) for var, stat in product(variables, stats)
    ]

    datasets = []
    for metadata in metadatas:  # standard_name ?
        data = analysis_service.get_data(metadata)
        ds = grid.as_xarray(data)
        datasets.append(ds)

    #TODO: return this as a list of tuples to keep data and metadata together?
    return datasets, metadatas
