from pathlib import Path
from dataclasses import dataclass
import json

STATIC_METADATA_PATH = Path(__file__).parent / "static_metadata"
TOAR_VARIABLES_METADATA_PATH = STATIC_METADATA_PATH / "toar_variables.json"
GLOABAL_CF_ATTRIBUTES_PATH = STATIC_METADATA_PATH / "global_cf_attributes.json"


def load_global_cf_attributes():
    """loading of global 

    Loads global metadata like details on the TOAD DB, license, and CF convention.
    """
    with open(GLOABAL_CF_ATTRIBUTES_PATH, "r") as f:
        return json.load(f)


@dataclass
class TOARVariable:
    """access to available variables of the TOAR database.

    This class knows of all variables of the TOAR database supported by this module.
    An instance allows direct access to the properties and metadata according to CF of this variable.
    """

    vars = None
    """available variables from the TOAR database."""
    
    name: str
    longname: str
    displayname: str
    cf_standardname: str
    units: str
    chemical_formula: str
    id: int

    @classmethod
    def load_toar_variables(cls) -> list["TOARVariable"]:
        """load available variables 
        
        Is executed by loading the TOARVariable class
        """
        with open(TOAR_VARIABLES_METADATA_PATH, "r") as f:
            variables = json.load(f)

        cls.vars = [TOARVariable(**var) for var in variables]

    @classmethod
    def get(cls, name: str) -> "TOARVariable":
        """searches the known variables for the requested variable
        
        Parameters:
        ----------
        name:
            name of the variable following the CF convention.
            For each variable first, the cf_standardname is checked and afterwards the name field.

        return:
            provides direct access to the meta data of the selected variable
        """
        #TODO: update README
        try:
            return cls.get_from_cf_standardname(standard_name=name)
        except ValueError as e:
            pass
        try:
            return cls.get_from_name(name=name)
        except ValueError as e:
            raise ValueError(f"TOAR Database contains no variable with cf_standardname or name '{name}'")
    @classmethod
    def get_from_cf_standardname(cls, standard_name: str) -> "TOARVariable":
        """searches the known variables for the given cf_standardname
        
        Parameters:
        ----------
        standard_name:
            name of the variable following the CF convention.

        return:
            provides direct access to the meta data of the selected variable
        """
        if standard_name == "" or standard_name == None:
            raise ValueError("No name provided for variable.")
        for var in cls.vars:
            if var.standard_name == standard_name:
                return var
        else:
            raise ValueError(f"TOAR Database contains no variable with cf_standardname {standard_name}")
    @classmethod
    def get_from_name(cls, name: str) -> "TOARVariable":
        """searches the known variables for the given name
        
        Parameters:
        ----------
        name:
            name of the variable according to the TOAR database config

        return:
            provides direct access to the meta data of the selected variable
        """
        if name == "" or name == None:
            raise ValueError("No name provided for variable.")
        for var in cls.vars:
            if var.name==name:
                return var
        else:
            raise ValueError(f"TOAR Database contains no variable {name}")

    @property
    def toar_id(self):
        """get toar ID; same as id
        """
        return self.id

    @property
    def standard_name(self):
        """alias to get cf_standardname
        """
        return self.cf_standardname

    @property
    def long_name(self):
        """alias to get longname
        """
        return self.longname


TOARVariable.load_toar_variables()
global_cf_attributes = load_global_cf_attributes()
